#include <iostream>
#include <list>
#include <sys/types.h>
#include <sys/time.h>

using namespace std;

#include "CIoInitiator.h"
#include "IoCreator.h"
#include "statitics.h"

#define NR_ARGS 7

//
// Description: 
// this tester is used to commence reads from one device 
//  in different offsets in the disk in a contiguos manner.
//  Granualrity of the reads is  the buffer size.
//  Total of size to read is the size of the stream multiplied by the number of streams
//  Size of Disk is used to frame the offsets within the disk size. 
//

CIoInitiatorFactory f;

void usage()
{
    printf("usage is:\n\t -p<device name>\n\t -s<stream size>\n\t -b<buffer size>\n\t -n<no. streams>\n\t -d<disk size> ");
}

int validate_args(){


 if  ( Streams<=0 ){
	 printf("Streams no. %d is not valid\n",Streams);
	 return -1;
 }
   
  if ( StreamSizeMB==0  || StreamSizeMB >= 100000 ){
	  printf("Stream Size  %d is not valid\n",StreamSizeMB);
	  return -1;		
 }

  if (  BufferSizeKB==0  ||  BufferSizeKB >= 104857600 ){
	  printf("Buffer Size  %d is not valid\n",BufferSizeKB);
	  return -1;		
  }

  if (diskSizeBytesMB <= 0 ) {
	  printf("disk Size  %d MB is not valid\n",diskSizeBytesMB);
	  return -1;
  }


  return 0;
}

extern bool bSort;


int main (int argc, char *argv[])
{
	list<CReader> readers_list;
 	
   
	int i = 1 ;
	string deviceName;

	if ( take_options(argc,argv,deviceName) < 0 || validate_args() < 0 ){

		usage();
		return -1;
	}

	i++;

	printf( "\tSize of Buffer %d KB\n\t" , BufferSizeKB );
	printf( "Size of Stream %d MB\n\t" , StreamSizeMB);
	printf( "Number Of Streams %d\n\t" , Streams ) ;
	printf( "Device Name %s\n\t",deviceName.c_str());
	printf( "Size of disk %d GB\n\t",(diskSizeBytesMB >> 10));

	if (bSort){
		printf("With sorting positions\n" );
	}else
		printf("Without sorting positions\n");


	// now create the readers
	list<CReader>::iterator iter;
	
	for (i=0;i<Streams;i++) {

		struct timeval rnd;
		gettimeofday(&rnd,0);
		unsigned long long randomLocationOnDiskMB  = ( (unsigned long long)rnd.tv_usec*rand() * (BufferSizeKB<<10) )%diskSizeBytesMB  ;
		cout <<"Stream[" << i << "] starting position is " << (randomLocationOnDiskMB>>10) <<" GB" << endl;  
		CReader* o = (CReader*)f.Create(BufferSizeKB<<10,randomLocationOnDiskMB<<20);
		readers_list.push_back(*o);
	}

	if (bSort){
		readers_list.sort();
	}

	for (iter=readers_list.begin();iter!=readers_list.end();++iter) {
		cout << (*iter).getCurpos() << endl;
	}

	// now initiates a sequential reads all over the disk

	for (iter=readers_list.begin();iter!=readers_list.end();++iter) {

		
		CIoInitiator* o = &(*iter);
		if(!o->open(deviceName,1)){

			cout << "failed to open"  << deviceName << endl;
			return -1;
		}
	}

	cout << "press enter to run test."; 
	getchar();

	unsigned long long totBytes = ((unsigned long long)StreamSizeMB<<20) *(unsigned long long)Streams ;

	gettimeofday(&ts,0);

	// new read up to end
	while(curReadBytes<totBytes){
		
		for (iter=readers_list.begin();iter!=readers_list.end();++iter) {

			CIoInitiator* o = &(*iter);
			if( !o->act() ){
				perror("");
				return -1;
			}
			curReadBytes+=(BufferSizeKB<<10);
		}
	}


	gettimeofday(&te,0);

	for (iter=readers_list.begin();iter!=readers_list.end();++iter) {
		io.push_back(&(*iter));

	}
	statistics();
	return(0);
}

